@@ -1,3 +1,5 @@
+//! Types.
+
 use fallible_iterator::FallibleIterator;
 use postgres_protocol;
 use postgres_protocol::types::{self, ArrayDimension};
@@ -6,11 +8,13 @@ use std::error::Error;
 use std::fmt;
 use std::sync::Arc;
 
+use types::type_gen::{Inner, Other};
+
 #[doc(inline)]
 pub use postgres_protocol::Oid;
 
-pub use self::type_gen::Type;
-pub use self::special::{Date, Timestamp};
+pub use types::type_gen::consts::*;
+pub use types::special::{Date, Timestamp};
 
 /// Generates a simple implementation of `ToSql::accepts` which accepts the
 /// types passed to it.
@@ -47,11 +51,13 @@ macro_rules! to_sql_checked {
 // WARNING: this function is not considered part of this crate's public API.
 // It is subject to change at any time.
 #[doc(hidden)]
-pub fn __to_sql_checked<T>(v: &T,
-                           ty: &Type,
-                           out: &mut Vec<u8>)
-                           -> Result<IsNull, Box<Error + Sync + Send>>
-    where T: ToSql
+pub fn __to_sql_checked<T>(
+    v: &T,
+    ty: &Type,
+    out: &mut Vec<u8>,
+) -> Result<IsNull, Box<Error + Sync + Send>>
+where
+    T: ToSql,
 {
     if !T::accepts(ty) {
         return Err(Box::new(WrongType(ty.clone())));
@@ -73,10 +79,68 @@ mod serde_json;
 mod chrono;
 #[cfg(feature = "with-eui48")]
 mod eui48;
+#[cfg(feature = "with-geo")]
+mod geo;
 
 mod special;
 mod type_gen;
 
+/// A Postgres type.
+#[derive(PartialEq, Eq, Clone, Debug)]
+pub struct Type(Inner);
+
+impl fmt::Display for Type {
+    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+        match self.schema() {
+            "public" | "pg_catalog" => {}
+            schema => write!(fmt, "{}.", schema)?,
+        }
+        fmt.write_str(self.name())
+    }
+}
+
+impl Type {
+    // WARNING: this is not considered public API
+    #[doc(hidden)]
+    pub fn _new(name: String, oid: Oid, kind: Kind, schema: String) -> Type {
+        Type(Inner::Other(Arc::new(Other {
+            name: name,
+            oid: oid,
+            kind: kind,
+            schema: schema,
+        })))
+    }
+
+    /// Returns the `Type` corresponding to the provided `Oid` if it
+    /// corresponds to a built-in type.
+    pub fn from_oid(oid: Oid) -> Option<Type> {
+        Inner::from_oid(oid).map(Type)
+    }
+
+    /// Returns the OID of the `Type`.
+    pub fn oid(&self) -> Oid {
+        self.0.oid()
+    }
+
+    /// Returns the kind of this type.
+    pub fn kind(&self) -> &Kind {
+        self.0.kind()
+    }
+
+    /// Returns the schema of this type.
+    pub fn schema(&self) -> &str {
+        match self.0 {
+            Inner::Other(ref u) => &u.schema,
+            _ => "pg_catalog",
+        }
+    }
+
+    /// Returns the name of this type.
+    pub fn name(&self) -> &str {
+        self.0.name()
+    }
+}
+
 /// Represents the kind of a Postgres type.
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum Kind {
@@ -127,63 +191,6 @@ impl Field {
     }
 }
 
-/// Information about an unknown type.
-#[derive(PartialEq, Eq, Clone)]
-pub struct Other(Arc<OtherInner>);
-
-impl fmt::Debug for Other {
-    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
-        fmt.debug_struct("Other")
-            .field("name", &self.0.name)
-            .field("oid", &self.0.oid)
-            .field("kind", &self.0.kind)
-            .field("schema", &self.0.schema)
-            .finish()
-    }
-}
-
-#[derive(PartialEq, Eq)]
-struct OtherInner {
-    name: String,
-    oid: Oid,
-    kind: Kind,
-    schema: String,
-}
-
-impl Other {
-    #[doc(hidden)]
-    pub fn new(name: String, oid: Oid, kind: Kind, schema: String) -> Other {
-        Other(Arc::new(OtherInner {
-            name: name,
-            oid: oid,
-            kind: kind,
-            schema: schema,
-        }))
-    }
-}
-
-impl Other {
-    /// The name of the type.
-    pub fn name(&self) -> &str {
-        &self.0.name
-    }
-
-    /// The OID of this type.
-    pub fn oid(&self) -> Oid {
-        self.0.oid
-    }
-
-    /// The kind of this type.
-    pub fn kind(&self) -> &Kind {
-        &self.0.kind
-    }
-
-    /// The schema of this type.
-    pub fn schema(&self) -> &str {
-        &self.0.schema
-    }
-}
-
 /// An error indicating that a `NULL` Postgres value was passed to a `FromSql`
 /// implementation that does not support `NULL` values.
 #[derive(Debug, Clone, Copy)]
@@ -208,9 +215,11 @@ pub struct WrongType(Type);
 
 impl fmt::Display for WrongType {
     fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
-        write!(fmt,
-               "cannot convert to or from a Postgres value of type `{}`",
-               self.0)
+        write!(
+            fmt,
+            "cannot convert to or from a Postgres value of type `{}`",
+            self.0
+        )
     }
 }
 
@@ -234,19 +243,19 @@ impl WrongType {
 /// The following implementations are provided by this crate, along with the
 /// corresponding Postgres types:
 ///
-/// | Rust type                         | Postgres type(s)                     |
-/// |-----------------------------------|--------------------------------------|
-/// | `bool`                            | BOOL                                 |
-/// | `i8`                              | "char"                               |
-/// | `i16`                             | SMALLINT, SMALLSERIAL                |
-/// | `i32`                             | INT, SERIAL                          |
-/// | `u32`                             | OID                                  |
-/// | `i64`                             | BIGINT, BIGSERIAL                    |
-/// | `f32`                             | REAL                                 |
-/// | `f64`                             | DOUBLE PRECISION                     |
-/// | `String`                          | VARCHAR, CHAR(n), TEXT, CITEXT, NAME |
-/// | `Vec<u8>`                         | BYTEA                                |
-/// | `HashMap<String, Option<String>>` | HSTORE                               |
+/// | Rust type                         | Postgres type(s)                              |
+/// |-----------------------------------|-----------------------------------------------|
+/// | `bool`                            | BOOL                                          |
+/// | `i8`                              | "char"                                        |
+/// | `i16`                             | SMALLINT, SMALLSERIAL                         |
+/// | `i32`                             | INT, SERIAL                                   |
+/// | `u32`                             | OID                                           |
+/// | `i64`                             | BIGINT, BIGSERIAL                             |
+/// | `f32`                             | REAL                                          |
+/// | `f64`                             | DOUBLE PRECISION                              |
+/// | `String`                          | VARCHAR, CHAR(n), TEXT, CITEXT, NAME, UNKNOWN |
+/// | `Vec<u8>`                         | BYTEA                                         |
+/// | `HashMap<String, Option<String>>` | HSTORE                                        |
 ///
 /// In addition, some implementations are provided for types in third party
 /// crates. These are disabled by default; to opt into one of these
@@ -260,7 +269,7 @@ impl WrongType {
 /// | `serde_json::Value`             | JSON, JSONB                         |
 /// | `time::Timespec`                | TIMESTAMP, TIMESTAMP WITH TIME ZONE |
 /// | `chrono::NaiveDateTime`         | TIMESTAMP                           |
-/// | `chrono::DateTime<UTC>`         | TIMESTAMP WITH TIME ZONE            |
+/// | `chrono::DateTime<Utc>`         | TIMESTAMP WITH TIME ZONE            |
 /// | `chrono::DateTime<Local>`       | TIMESTAMP WITH TIME ZONE            |
 /// | `chrono::DateTime<FixedOffset>` | TIMESTAMP WITH TIME ZONE            |
 /// | `chrono::NaiveDate`             | DATE                                |
@@ -286,9 +295,7 @@ pub trait FromSql: Sized {
     ///
     /// The caller of this method is responsible for ensuring that this type
     /// is compatible with the Postgres `Type`.
-    fn from_sql(ty: &Type,
-                raw: &[u8])
-                -> Result<Self, Box<Error + Sync + Send>>;
+    fn from_sql(ty: &Type, raw: &[u8]) -> Result<Self, Box<Error + Sync + Send>>;
 
     /// Creates a new value of this type from a `NULL` SQL value.
     ///
@@ -304,9 +311,7 @@ pub trait FromSql: Sized {
 
     /// A convenience function that delegates to `from_sql` and `from_sql_null` depending on the
     /// value of `raw`.
-    fn from_sql_nullable(ty: &Type,
-                         raw: Option<&[u8]>)
-                         -> Result<Self, Box<Error + Sync + Send>> {
+    fn from_sql_nullable(ty: &Type, raw: Option<&[u8]>) -> Result<Self, Box<Error + Sync + Send>> {
         match raw {
             Some(raw) => Self::from_sql(ty, raw),
             None => Self::from_sql_null(ty),
@@ -319,9 +324,7 @@ pub trait FromSql: Sized {
 }
 
 impl<T: FromSql> FromSql for Option<T> {
-    fn from_sql(ty: &Type,
-                raw: &[u8])
-                -> Result<Option<T>, Box<Error + Sync + Send>> {
+    fn from_sql(ty: &Type, raw: &[u8]) -> Result<Option<T>, Box<Error + Sync + Send>> {
         <T as FromSql>::from_sql(ty, raw).map(Some)
     }
 
@@ -335,9 +338,7 @@ impl<T: FromSql> FromSql for Option<T> {
 }
 
 impl<T: FromSql> FromSql for Vec<T> {
-    fn from_sql(ty: &Type,
-                raw: &[u8])
-                -> Result<Vec<T>, Box<Error + Sync + Send>> {
+    fn from_sql(ty: &Type, raw: &[u8]) -> Result<Vec<T>, Box<Error + Sync + Send>> {
         let member_type = match *ty.kind() {
             Kind::Array(ref member) => member,
             _ => panic!("expected array type"),
@@ -348,7 +349,8 @@ impl<T: FromSql> FromSql for Vec<T> {
             return Err("array contains too many dimensions".into());
         }
 
-        array.values()
+        array
+            .values()
             .and_then(|v| T::from_sql_nullable(member_type, v))
             .collect()
     }
@@ -362,13 +364,11 @@ impl<T: FromSql> FromSql for Vec<T> {
 }
 
 impl FromSql for Vec<u8> {
-    fn from_sql(_: &Type,
-                raw: &[u8])
-                -> Result<Vec<u8>, Box<Error + Sync + Send>> {
+    fn from_sql(_: &Type, raw: &[u8]) -> Result<Vec<u8>, Box<Error + Sync + Send>> {
         Ok(types::bytea_from_sql(raw).to_owned())
     }
 
-    accepts!(Type::Bytea);
+    accepts!(BYTEA);
 }
 
 impl FromSql for String {
@@ -378,8 +378,8 @@ impl FromSql for String {
 
     fn accepts(ty: &Type) -> bool {
         match *ty {
-            Type::Varchar | Type::Text | Type::Bpchar | Type::Name => true,
-            Type::Other(ref u) if u.name() == "citext" => true,
+            VARCHAR | TEXT | BPCHAR | NAME | UNKNOWN => true,
+            ref ty if ty.name() == "citext" => true,
             _ => false,
         }
     }
@@ -399,29 +399,27 @@ macro_rules! simple_from {
     }
 }
 
-simple_from!(bool, bool_from_sql, Type::Bool);
-simple_from!(i8, char_from_sql, Type::Char);
-simple_from!(i16, int2_from_sql, Type::Int2);
-simple_from!(i32, int4_from_sql, Type::Int4);
-simple_from!(u32, oid_from_sql, Type::Oid);
-simple_from!(i64, int8_from_sql, Type::Int8);
-simple_from!(f32, float4_from_sql, Type::Float4);
-simple_from!(f64, float8_from_sql, Type::Float8);
+simple_from!(bool, bool_from_sql, BOOL);
+simple_from!(i8, char_from_sql, CHAR);
+simple_from!(i16, int2_from_sql, INT2);
+simple_from!(i32, int4_from_sql, INT4);
+simple_from!(u32, oid_from_sql, OID);
+simple_from!(i64, int8_from_sql, INT8);
+simple_from!(f32, float4_from_sql, FLOAT4);
+simple_from!(f64, float8_from_sql, FLOAT8);
 
 impl FromSql for HashMap<String, Option<String>> {
-    fn from_sql(_: &Type,
-                raw: &[u8])
-                -> Result<HashMap<String, Option<String>>, Box<Error + Sync + Send>> {
+    fn from_sql(
+        _: &Type,
+        raw: &[u8],
+    ) -> Result<HashMap<String, Option<String>>, Box<Error + Sync + Send>> {
         types::hstore_from_sql(raw)?
             .map(|(k, v)| (k.to_owned(), v.map(str::to_owned)))
             .collect()
     }
 
     fn accepts(ty: &Type) -> bool {
-        match *ty {
-            Type::Other(ref u) if u.name() == "hstore" => true,
-            _ => false,
-        }
+        ty.name() == "hstore"
     }
 }
 
@@ -468,7 +466,7 @@ pub enum IsNull {
 /// | `serde_json::Value`             | JSON, JSONB                         |
 /// | `time::Timespec`                | TIMESTAMP, TIMESTAMP WITH TIME ZONE |
 /// | `chrono::NaiveDateTime`         | TIMESTAMP                           |
-/// | `chrono::DateTime<UTC>`         | TIMESTAMP WITH TIME ZONE            |
+/// | `chrono::DateTime<Utc>`         | TIMESTAMP WITH TIME ZONE            |
 /// | `chrono::DateTime<Local>`       | TIMESTAMP WITH TIME ZONE            |
 /// | `chrono::DateTime<FixedOffset>` | TIMESTAMP WITH TIME ZONE            |
 /// | `chrono::NaiveDate`             | DATE                                |
@@ -498,33 +496,32 @@ pub trait ToSql: fmt::Debug {
     /// The return value indicates if this value should be represented as
     /// `NULL`. If this is the case, implementations **must not** write
     /// anything to `out`.
-    fn to_sql(&self,
-              ty: &Type,
-              out: &mut Vec<u8>)
-              -> Result<IsNull, Box<Error + Sync + Send>>
-        where Self: Sized;
+    fn to_sql(&self, ty: &Type, out: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>>
+    where
+        Self: Sized;
 
     /// Determines if a value of this type can be converted to the specified
     /// Postgres `Type`.
-    fn accepts(ty: &Type) -> bool where Self: Sized;
+    fn accepts(ty: &Type) -> bool
+    where
+        Self: Sized;
 
     /// An adaptor method used internally by Rust-Postgres.
     ///
     /// *All* implementations of this method should be generated by the
     /// `to_sql_checked!()` macro.
-    fn to_sql_checked(&self,
-                      ty: &Type,
-                      out: &mut Vec<u8>)
-                      -> Result<IsNull, Box<Error + Sync + Send>>;
+    fn to_sql_checked(
+        &self,
+        ty: &Type,
+        out: &mut Vec<u8>,
+    ) -> Result<IsNull, Box<Error + Sync + Send>>;
 }
 
 impl<'a, T> ToSql for &'a T
-    where T: ToSql
+where
+    T: ToSql,
 {
-    fn to_sql(&self,
-              ty: &Type,
-              out: &mut Vec<u8>)
-              -> Result<IsNull, Box<Error + Sync + Send>> {
+    fn to_sql(&self, ty: &Type, out: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>> {
         (*self).to_sql(ty, out)
     }
 
@@ -536,10 +533,7 @@ impl<'a, T> ToSql for &'a T
 }
 
 impl<T: ToSql> ToSql for Option<T> {
-    fn to_sql(&self,
-              ty: &Type,
-              out: &mut Vec<u8>)
-              -> Result<IsNull, Box<Error + Sync + Send>> {
+    fn to_sql(&self, ty: &Type, out: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>> {
         match *self {
             Some(ref val) => val.to_sql(ty, out),
             None => Ok(IsNull::Yes),
@@ -554,10 +548,7 @@ impl<T: ToSql> ToSql for Option<T> {
 }
 
 impl<'a, T: ToSql> ToSql for &'a [T] {
-    fn to_sql(&self,
-              ty: &Type,
-              w: &mut Vec<u8>)
-              -> Result<IsNull, Box<Error + Sync + Send>> {
+    fn to_sql(&self, ty: &Type, w: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>> {
         let member_type = match *ty.kind() {
             Kind::Array(ref member) => member,
             _ => panic!("expected array type"),
@@ -568,17 +559,17 @@ impl<'a, T: ToSql> ToSql for &'a [T] {
             lower_bound: 1,
         };
 
-        types::array_to_sql(Some(dimension),
-                                 true,
-                                 member_type.oid(),
-                                 self.iter(),
-                                 |e, w| {
-                                     match e.to_sql(member_type, w)? {
-                                         IsNull::No => Ok(postgres_protocol::IsNull::No),
-                                         IsNull::Yes => Ok(postgres_protocol::IsNull::Yes),
-                                     }
-                                 },
-                                 w)?;
+        types::array_to_sql(
+            Some(dimension),
+            true,
+            member_type.oid(),
+            self.iter(),
+            |e, w| match e.to_sql(member_type, w)? {
+                IsNull::No => Ok(postgres_protocol::IsNull::No),
+                IsNull::Yes => Ok(postgres_protocol::IsNull::Yes),
+            },
+            w,
+        )?;
         Ok(IsNull::No)
     }
 
@@ -593,24 +584,18 @@ impl<'a, T: ToSql> ToSql for &'a [T] {
 }
 
 impl<'a> ToSql for &'a [u8] {
-    fn to_sql(&self,
-              _: &Type,
-              w: &mut Vec<u8>)
-              -> Result<IsNull, Box<Error + Sync + Send>> {
+    fn to_sql(&self, _: &Type, w: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>> {
         types::bytea_to_sql(*self, w);
         Ok(IsNull::No)
     }
 
-    accepts!(Type::Bytea);
+    accepts!(BYTEA);
 
     to_sql_checked!();
 }
 
 impl<T: ToSql> ToSql for Vec<T> {
-    fn to_sql(&self,
-              ty: &Type,
-              w: &mut Vec<u8>)
-              -> Result<IsNull, Box<Error + Sync + Send>> {
+    fn to_sql(&self, ty: &Type, w: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>> {
         <&[T] as ToSql>::to_sql(&&**self, ty, w)
     }
 
@@ -622,10 +607,7 @@ impl<T: ToSql> ToSql for Vec<T> {
 }
 
 impl ToSql for Vec<u8> {
-    fn to_sql(&self,
-              ty: &Type,
-              w: &mut Vec<u8>)
-              -> Result<IsNull, Box<Error + Sync + Send>> {
+    fn to_sql(&self, ty: &Type, w: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>> {
         <&[u8] as ToSql>::to_sql(&&**self, ty, w)
     }
 
@@ -637,18 +619,15 @@ impl ToSql for Vec<u8> {
 }
 
 impl<'a> ToSql for &'a str {
-    fn to_sql(&self,
-              _: &Type,
-              w: &mut Vec<u8>)
-              -> Result<IsNull, Box<Error + Sync + Send>> {
+    fn to_sql(&self, _: &Type, w: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>> {
         types::text_to_sql(*self, w);
         Ok(IsNull::No)
     }
 
     fn accepts(ty: &Type) -> bool {
         match *ty {
-            Type::Varchar | Type::Text | Type::Bpchar | Type::Name => true,
-            Type::Other(ref u) if u.name() == "citext" => true,
+            VARCHAR | TEXT | BPCHAR | NAME | UNKNOWN => true,
+            ref ty if ty.name() == "citext" => true,
             _ => false,
         }
     }
@@ -657,10 +636,7 @@ impl<'a> ToSql for &'a str {
 }
 
 impl ToSql for String {
-    fn to_sql(&self,
-              ty: &Type,
-              w: &mut Vec<u8>)
-              -> Result<IsNull, Box<Error + Sync + Send>> {
+    fn to_sql(&self, ty: &Type, w: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>> {
         <&str as ToSql>::to_sql(&&**self, ty, w)
     }
 
@@ -689,29 +665,26 @@ macro_rules! simple_to {
     }
 }
 
-simple_to!(bool, bool_to_sql, Type::Bool);
-simple_to!(i8, char_to_sql, Type::Char);
-simple_to!(i16, int2_to_sql, Type::Int2);
-simple_to!(i32, int4_to_sql, Type::Int4);
-simple_to!(u32, oid_to_sql, Type::Oid);
-simple_to!(i64, int8_to_sql, Type::Int8);
-simple_to!(f32, float4_to_sql, Type::Float4);
-simple_to!(f64, float8_to_sql, Type::Float8);
+simple_to!(bool, bool_to_sql, BOOL);
+simple_to!(i8, char_to_sql, CHAR);
+simple_to!(i16, int2_to_sql, INT2);
+simple_to!(i32, int4_to_sql, INT4);
+simple_to!(u32, oid_to_sql, OID);
+simple_to!(i64, int8_to_sql, INT8);
+simple_to!(f32, float4_to_sql, FLOAT4);
+simple_to!(f64, float8_to_sql, FLOAT8);
 
 impl ToSql for HashMap<String, Option<String>> {
-    fn to_sql(&self,
-              _: &Type,
-              w: &mut Vec<u8>)
-              -> Result<IsNull, Box<Error + Sync + Send>> {
-        types::hstore_to_sql(self.iter().map(|(k, v)| (&**k, v.as_ref().map(|v| &**v))), w)?;
+    fn to_sql(&self, _: &Type, w: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>> {
+        types::hstore_to_sql(
+            self.iter().map(|(k, v)| (&**k, v.as_ref().map(|v| &**v))),
+            w,
+        )?;
         Ok(IsNull::No)
     }
 
     fn accepts(ty: &Type) -> bool {
-        match *ty {
-            Type::Other(ref u) if u.name() == "hstore" => true,
-            _ => false,
-        }
+        ty.name() == "hstore"
     }
 
     to_sql_checked!();
