@@ -4,7 +4,7 @@ use self::time::Timespec;
 use std::error::Error;
 use postgres_protocol::types;
 
-use types::{Type, FromSql, ToSql, IsNull};
+use types::{Type, FromSql, ToSql, IsNull, TIMESTAMP, TIMESTAMPTZ};
 
 const USEC_PER_SEC: i64 = 1_000_000;
 const NSEC_PER_USEC: i64 = 1_000;
@@ -13,9 +13,7 @@ const NSEC_PER_USEC: i64 = 1_000;
 const TIME_SEC_CONVERSION: i64 = 946684800;
 
 impl FromSql for Timespec {
-    fn from_sql(_: &Type,
-                raw: &[u8])
-                -> Result<Timespec, Box<Error + Sync + Send>> {
+    fn from_sql(_: &Type, raw: &[u8]) -> Result<Timespec, Box<Error + Sync + Send>> {
         let t = types::timestamp_from_sql(raw)?;
         let mut sec = t / USEC_PER_SEC + TIME_SEC_CONVERSION;
         let mut usec = t % USEC_PER_SEC;
@@ -28,19 +26,16 @@ impl FromSql for Timespec {
         Ok(Timespec::new(sec, (usec * NSEC_PER_USEC) as i32))
     }
 
-    accepts!(Type::Timestamp, Type::Timestamptz);
+    accepts!(TIMESTAMP, TIMESTAMPTZ);
 }
 
 impl ToSql for Timespec {
-    fn to_sql(&self,
-              _: &Type,
-              w: &mut Vec<u8>)
-              -> Result<IsNull, Box<Error + Sync + Send>> {
+    fn to_sql(&self, _: &Type, w: &mut Vec<u8>) -> Result<IsNull, Box<Error + Sync + Send>> {
         let t = (self.sec - TIME_SEC_CONVERSION) * USEC_PER_SEC + self.nsec as i64 / NSEC_PER_USEC;
         types::timestamp_to_sql(t, w);
         Ok(IsNull::No)
     }
 
-    accepts!(Type::Timestamp, Type::Timestamptz);
+    accepts!(TIMESTAMP, TIMESTAMPTZ);
     to_sql_checked!();
 }
